#include "stdafx.h"
#include <gl\gl.h>
#include <gl\glu.h>
#include <mmsystem.h>
#include "resource.h"
#include "textures.h"
#include "classes.h"

HDC hdc;
HWND hDlg;
HFONT hFont;
GameObject *ActiveObject = NULL, *NewObject = NULL;
char AppName[] = "IT shooting";
char ClassName[] = "ITShoot";

void SetDCPixelFormat(HDC hdc)
{
	static PIXELFORMATDESCRIPTOR pfd =
	{	sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
		| PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		32,
		0, 0, 0, 0, 0, 0,
		0, 0,
		0, 0, 0, 0, 0,
		32,
		0,
		0,
		PFD_MAIN_PLANE,
		0,
		0, 0, 0
	};

	int pixelFormat = ChoosePixelFormat(hdc, &pfd);
	SetPixelFormat(hdc, pixelFormat, &pfd);
}

void InitGL()
{
	GLfloat lightAmbient[] = {0.1f, 0.1f, 0.1f, 1.0f};
	GLfloat lightDiffuse[] = {1.0f, 1.0f, 1.0f, 1.0f};

	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
    glEnable(GL_LIGHT0);
	glFrontFace(GL_CCW);
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	glShadeModel(GL_SMOOTH);
	glDepthFunc(GL_LEQUAL);
	glDisable(GL_DEPTH_TEST);
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}

void DrawScene(HDC hDC, HWND hWnd)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	ActiveObject->Draw();

	glFlush();
	SwapBuffers(hDC);
}

LONG WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static HGLRC hrc;

	switch(msg)
	{
	    case WM_CREATE:
			hdc = GetDC(hWnd);
            SetDCPixelFormat(hdc);
            hrc = wglCreateContext(hdc);
			wglMakeCurrent(hdc, hrc);
			InitGL();
			GenTextures();
			ActiveObject = CreateObjects();
			ActiveObject->Start(hWnd);
			SendMessage(hDlg, WM_CLOSE, 0, 0);
			break;

		case WM_SIZE:
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(30.0, 4.0 / 3.0, 0.01, 16.0);
			glViewport(0, 0, (GLsizei)LOWORD(lParam), (GLsizei)HIWORD(lParam));
			break;

		case WM_DESTROY:
			ActiveObject->Stop();
			DeleteObjects();
			wglMakeCurrent(NULL, NULL);
			wglDeleteContext(hrc);
			ReleaseDC(hWnd, hdc);
			PostQuitMessage(0);
			break;

		case MM_MCINOTIFY:
		case WM_LBUTTONDOWN:
		case WM_RBUTTONDOWN:
		case WM_MOUSEMOVE:
		case WM_KEYDOWN:
		case WM_KEYUP:
			ActiveObject->HandleEvent(hWnd, msg, wParam, lParam);
			break;

		default:
			return DefWindowProc(hWnd, msg, wParam, lParam);
	}

	return 0;
}

BOOL CALLBACK DialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG: 
			LOGFONT logfont;
			logfont.lfHeight = -32;
			logfont.lfWidth = 0;
			logfont.lfEscapement = 0;
			logfont.lfOrientation = 0;
			logfont.lfWeight = FW_BOLD;
			logfont.lfItalic = FALSE;
			logfont.lfUnderline = FALSE;
			logfont.lfStrikeOut = FALSE;
			logfont.lfCharSet = ANSI_CHARSET;
			logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
			logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
			logfont.lfQuality = DEFAULT_QUALITY;
			logfont.lfPitchAndFamily = DEFAULT_PITCH;
			strcpy(logfont.lfFaceName,"Times New Roman Cyr");
			hFont = CreateFontIndirect(&logfont);
			SendMessage(GetDlgItem(hWnd, IDC_STATIC), WM_SETFONT, (DWORD)hFont, 1);
			return TRUE;
		case WM_DESTROY:
			DeleteObject(hFont);
			return TRUE;
		default: return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}

	return FALSE;
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	WNDCLASS wc;
	HWND hWnd;
	MSG msg;
	BOOL ssaver;

	SystemParametersInfo(SPI_GETSCREENSAVEACTIVE, FALSE, &ssaver, FALSE);
	SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, FALSE, NULL, FALSE);

	hDlg = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_LOADING), 0, DialogProc);
	ShowWindow(hDlg, SW_SHOWNORMAL);
	SendMessage(hDlg, WM_INITDIALOG, 0, 0);

	wc.style = 0;
	wc.lpfnWndProc =(WNDPROC)WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_SHOOT));
	wc.hCursor = LoadCursor(hInstance, MAKEINTRESOURCE(IDC_CURSOR));
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = (LPCTSTR)0;
	wc.lpszClassName = ClassName;

	RegisterClass(&wc);

	hWnd = CreateWindow(ClassName, AppName,
						WS_POPUP | WS_VISIBLE,
						0, 0, GetSystemMetrics(SM_CXSCREEN),
						GetSystemMetrics(SM_CYSCREEN), HWND_DESKTOP, NULL, hInstance, NULL);

	UpdateWindow(hWnd);

    while(1)
    {
		if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if(!GetMessage(&msg, NULL, 0, 0)) break;
			TranslateMessage(&msg); 
			DispatchMessage(&msg);
		}
	    else if (ActiveObject)
		{
			ActiveObject->UpdateObject(hWnd);
			if (ActiveObject->Done(NewObject))
			{
				ActiveObject->Stop();
				if (!NewObject)
				{
					SendMessage(hWnd, WM_CLOSE, 0, 0);
					continue;
				}
				ActiveObject = NewObject;
				ActiveObject->Start(hWnd);
			}
			DrawScene(hdc, hWnd);
		}
    }

	SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, ssaver, NULL, FALSE);

	return msg.wParam;
}
