#include <fstream>
#include <windows.h>
#include <mmsystem.h>
#include <math.h>
#include "classes.h"
#include "textures.h"

using namespace std;

const float M_PI = 3.14159265358f;
const float PI180 = M_PI / 180.0f;
const char overstr[] = "GAMEOVER";
const float c = 2 * M_PI / COIN_EDGES;

Intro *intro;
Menu *menu;
Preface *preface;
Shooting *shooting;
WinGame *wingame;
GameOver *gameover;
int chapter = 0;
GLYPHMETRICSFLOAT *agmf;

float *sin_table;
float *cos_table;

GLuint ass[NUM_ASS];
GLuint doz[NUM_DOZ];
GLuint prof[NUM_PROF];

GameObject *CreateObjects()
{
	HFONT hFont;
	LOGFONT logfont;
	logfont.lfHeight = -10;
	logfont.lfWidth = 0;
	logfont.lfEscapement = 0;
	logfont.lfOrientation = 0;
	logfont.lfWeight = FW_BOLD;
	logfont.lfItalic = FALSE;
	logfont.lfUnderline = FALSE;
	logfont.lfStrikeOut = FALSE;
	logfont.lfCharSet = ANSI_CHARSET;
	logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
	logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	logfont.lfQuality = DEFAULT_QUALITY;
	logfont.lfPitchAndFamily = DEFAULT_PITCH;
	strcpy(logfont.lfFaceName,"Arial Cyr");
	hFont = CreateFontIndirect(&logfont);
	SelectObject (wglGetCurrentDC(), hFont); 
	agmf = new GLYPHMETRICSFLOAT[256];
	wglUseFontOutlines(wglGetCurrentDC(), 0, 256, 1000, 0.0f, 0.2f, WGL_FONT_POLYGONS, agmf);
	DeleteObject(hFont);
	char buf[32];
	int i = 0;
	for (; i < NUM_ASS; i++)
	{
		wsprintf(buf, "textures\\ass%d.bmp", i + 1);
		ass[i] = LoadTexture(buf, 128, 128);
	}
	for (i = 0; i < NUM_DOZ; i++)
	{
		wsprintf(buf, "textures\\doz%d.bmp", i + 1);
		doz[i] = LoadTexture(buf, 128, 128);
	}
	for (i = 0; i < NUM_PROF; i++)
	{
		wsprintf(buf, "textures\\prof%d.bmp", i + 1);
		prof[i] = LoadTexture(buf, 128, 128);
	}
	cos_table = new float[COIN_EDGES];
	sin_table = new float[COIN_EDGES];
	for (i = 0; i < COIN_EDGES; i++)
	{
		cos_table[i] = float(0.5 * cos(i * c));
		sin_table[i] = float(0.5 * sin(i * c));
	}
	intro = new Intro();
	menu = new Menu();
	preface = new Preface();
	shooting = new Shooting();
	wingame = new WinGame();
	gameover = new GameOver();
	return intro;
}

void DeleteObjects()
{
	delete cos_table;
	delete sin_table;
	delete agmf;
	delete shooting;
	delete preface;
	delete menu;
	delete intro;
}

void CenterMouse(HWND hWnd)
{
	RECT r;
	POINT p;
	HDC dc = CreateDC("DISPLAY", NULL, NULL, NULL);
	int w = GetDeviceCaps(dc, HORZRES) - 1;
	int h = GetDeviceCaps(dc, VERTRES) - 1;
	DeleteDC(dc);
	GetClientRect(hWnd, &r);
	p.x = (r.left + r.right) >> 1;
	p.y = (r.top + r.bottom) >> 1;
	ClientToScreen(hWnd, &p);
	mouse_event(MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE, 65535 * p.x / w, 65535 * p.y / h, 0, 0);
}

GameObject::GameObject(char *theme)
{
	char cmd[256];
	wsprintf(cmd, "open music\\%s.mid type sequencer alias %s", theme, theme);
	mciSendString(cmd, NULL, 0, 0);
	midi_id = mciGetDeviceID(theme);
}

GameObject::~GameObject()
{
	mciSendString("close theme wait", NULL, 0, 0);
}

void GameObject::HandleEvent(HWND hWnd, UINT msg, WPARAM, LPARAM lParam)
{
	if (msg != MM_MCINOTIFY) return;
	mciSendCommand(lParam, MCI_SEEK, MCI_SEEK_TO_START, NULL);
	if ((MCIDEVICEID)lParam == midi_id)
	{
		MCI_PLAY_PARMS p;
		p.dwCallback = (unsigned long)hWnd;
		mciSendCommand(midi_id, MCI_PLAY, MCI_NOTIFY, (unsigned long)&p);
	}
}

void GameObject::UpdateObject(HWND)
{
	DWORD last_t = t;
	t = timeGetTime() - time;
	dt = t - last_t;
	alpha += fade_dir * (int)dt * FADE_TIME;
	if (alpha <= 0 && fade_dir < 0)
	{
		alpha = 0;
		fade_dir = 0;
	}
	else if (alpha >= 1 && fade_dir > 0)
	{
		alpha = 1;
		fade_dir = 0;
	}
}

void GameObject::Start(HWND hWnd)
{
	MCI_PLAY_PARMS p;
	done = false;
	time = timeGetTime();
	t = 0;
	p.dwCallback = (unsigned int)hWnd;
	mciSendCommand(midi_id, MCI_PLAY, MCI_NOTIFY, (unsigned long)&p);
	FadeIn();
}

void GameObject::Stop()
{
	mciSendCommand(midi_id, MCI_STOP, 0, NULL);
	mciSendCommand(midi_id, MCI_SEEK, MCI_SEEK_TO_START, NULL);
}

void GameObject::FadeOut()
{
	done = true;
	alpha = 1;
	fade_dir = -1;
}

void GameObject::FadeIn()
{
	alpha = 0;
	fade_dir = 1;
}

void GameObject::Draw()
{
	if (!fade_dir) return;
	glLoadIdentity();
	glEnable(GL_BLEND);
	glBlendFunc(GL_DST_COLOR, GL_ZERO);
	glDisable(GL_TEXTURE_2D);
	glColor3f(alpha, alpha, alpha);
	glBegin(GL_QUADS);
		glVertex3i(-1, 1, -1);
		glVertex3i(-1, -1, -1);
		glVertex3i(1, -1, -1);
		glVertex3i(1, 1, -1);
	glEnd();
	glColor3f(1, 1, 1);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

Sprite::Sprite(char *name, int w, int h)
{
	char path1[32] = "sprites\\";
	char path2[32] = "masks\\";
	strcat(path1, name);
	strcat(path2, name);
	sprite = LoadTexture(path1, w, h);
	mask = LoadTexture(path2, w, h);
	z = -1.0;
	t = 0.0;
}

void Sprite::Draw(float x, float y, float w, float h)
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);
	glBindTexture(GL_TEXTURE_2D, mask);
	glBegin(GL_QUADS);
		glTexCoord2i(0, 1);
		glVertex3f(x, y, z);
		glTexCoord1i(0);
		glVertex3f(x, y - h, z);
		glTexCoord1i(1);
		glVertex3f(x + w, y - h, z);
		glTexCoord2i(1, 1);
		glVertex3f(x + w, y, z);
	glEnd();
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
	glBindTexture(GL_TEXTURE_2D, sprite);
	glBegin(GL_QUADS);
		glTexCoord2i(0, 1);
		glVertex3f(x, y, z);
		glTexCoord1i(0);
		glVertex3f(x, y - h, z);
		glTexCoord1i(1);
		glVertex3f(x + w, y - h, z);
		glTexCoord2i(1, 1);
		glVertex3f(x + w, y, z);
	glEnd();
	glDisable(GL_BLEND);
}

Menu::Menu(): GameObject("menu")
{
	pointer = new Sprite("menuptr.bmp", 64, 64);
	background = LoadTexture("textures\\menuback.bmp", 256, 256);
	mciSendString("open sounds\\menuselect.wav type waveaudio alias menuselect", NULL, 0, 0);
	mciSendString("open sounds\\menupress.wav type waveaudio alias menupress", NULL, 0, 0);
}

Menu::~Menu()
{
	mciSendString("close menuselect wait", NULL, 0, 0);
	mciSendString("close menupress wait", NULL, 0, 0);
	delete pointer;
}

void Menu::HandleEvent(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	float y_;
	GameObject::HandleEvent(hWnd, msg, wParam, lParam);
	if (msg == WM_LBUTTONDOWN && current_item >= 0)
	{
		mciSendString("play menupress notify", NULL, 0, hWnd);
		switch (current_item)
		{
			case 0: 
				chapter = 0;
				FadeOut(); 
				break;
			case 1:
				next = NULL;
				FadeOut(); 
				break;
		}
	}
	else if (msg != WM_MOUSEMOVE) return;
	RECT r;
	GetClientRect(hWnd, &r);
	int h = r.bottom - r.top;
	int w = r.right - r.left;
	x = 0.54f * (LOWORD(lParam) - (w >> 1)) / h - 0.05f;
	y_ = 0.54f * ((h >> 1) - HIWORD(lParam)) / h;
	y = y_ + 0.05f;
	int last = current_item;
	if (y_ >= 0 && y_ < 0.125) current_item = 0;
	else if (y_ < 0 && y_ > -0.125) current_item = 1;
	else current_item = -1;
	if (current_item != -1)
	{
		y2 = 0.0625f - (current_item * 0.125f);
		if (last == -1) y1 = y2;
		if (last != current_item && current_item != -1) 
			mciSendString("play menuselect notify", NULL, 0, hWnd);
	}
}

void Menu::DrawString(float x, float y_, char *text, bool selected)
{
	float y = y_ + -agmf[(unsigned char)text[0]].gmfptGlyphOrigin.y + agmf[(unsigned char)text[0]].gmfBlackBoxY / 2;
	float w, q = 0, p = float(selected ? -9 : -10), a1, a2, v = t * 0.01f;
	int i, l;
	GLfloat diffuse[] = {1, 1, 0};
	GLYPHMETRICSFLOAT m;

	l = strlen(text);
	w = 0;
	for (i = 0; i < l; i++)
		w += agmf[(unsigned char)text[i]].gmfBlackBoxX + 0.1f;
	w -= 0.1f;

	glEnable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	for (i = 0; i < l; i++)
	{
		a1 = float(sin(v + i * M_PI / l) * 10);
		a2 = float(sin(v * 0.5 + i * M_PI / l) * 20);
		if (selected) 
		{
			q = float(sin(v + i * M_PI / 4) * 0.1);
			diffuse[1] = float(sin(v + i * M_PI / 4));
		}
		else
			diffuse[1] = 1;
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
		m = agmf[(unsigned char)text[i]];
		glPushMatrix();
		glTranslatef(x - w / 2, y + q, p); 
		glTranslatef(m.gmfptGlyphOrigin.x + m.gmfBlackBoxX / 2, 0, 0); 
		glRotatef(a1, 1, 0, 0);
		glRotatef(a2, 0, 1, 0);
		glTranslatef(-m.gmfptGlyphOrigin.x - m.gmfBlackBoxX / 2, 0, 0.1f); 
		x += m.gmfBlackBoxX + 0.1f;
		glCallList(1000 + (unsigned char)text[i]); 
		glPopMatrix();
	}
	glPopAttrib();
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
}

void Menu::Draw()
{
	glTranslatef(0, 0, float(sin(t * 0.005) - 1));
	glRotatef(t * 0.1f, 0, 0, 1);
	glBindTexture(GL_TEXTURE_2D, background);
	glBegin(GL_QUADS);
		glTexCoord2i(1, 1);
		glVertex3i(1, 1, -1);
		glTexCoord2i(0, 1);
		glVertex3i(-1, 1, -1);
		glTexCoord1i(0);
		glVertex3i(-1, -1, -1);
		glTexCoord1i(1);
		glVertex3i(1, -1, -1);
	glEnd();
	glLoadIdentity();
	DrawString(0, 0.5, "New Game", !current_item);
	DrawString(0, -0.5, "Exit to OS", current_item == 1);
	if (current_item >= 0)
	{
		glDisable(GL_TEXTURE_2D);
		glColor3f(float(sin(t * 0.01) * 0.25 + 0.5), 0, 0);
		glBegin(GL_TRIANGLES);
			glVertex3f(-0.31f, y1, -1);
			glVertex3f(-0.36f, y1 + 0.05f, -1);
			glVertex3f(-0.36f, y1 - 0.05f, -1);
			glVertex3f(0.31f, y1, -1);
			glVertex3f(0.36f, y1 - 0.05f, -1);
			glVertex3f(0.36f, y1 + 0.05f, -1);
		glEnd();
		glColor3f(1, 1, 1);
		glEnable(GL_TEXTURE_2D);
	}
	pointer->Draw(x, y, 0.1f, 0.1f);
	GameObject::Draw();
}

void Menu::UpdateObject(HWND hWnd)
{
	GameObject::UpdateObject(hWnd);
	y1 += (y2 - y1) * dt * 0.01f;
}

bool Menu::Done(GameObject *&new_obj)
{
	new_obj = next;
	return (done && !fade_dir);
}

void Menu::Start(HWND hWnd)
{
	GameObject::Start(hWnd);
	current_item = -1;
	next = preface;
	CenterMouse(hWnd);
}

void Menu::Stop()
{
	GameObject::Stop();
	mciSendString("stop menuselect wait", NULL, 0, 0);
	mciSendString("seek menuselect to start wait", NULL, 0, 0);
	mciSendString("stop menupress wait", NULL, 0, 0);
	mciSendString("seek menupress to start wait", NULL, 0, 0);
}

Preface::Preface(): GameObject("preface")
{
	rubles[0] = LoadTexture("textures\\1ruble.bmp", 256, 256);
	rubles[1] = LoadTexture("textures\\2rubles.bmp", 256, 256);
	rubles[2] = LoadTexture("textures\\5rubles.bmp", 256, 256);
	cointex[2] = LoadTexture("textures\\rubleside.bmp", 32, 64);
	background = LoadTexture("textures\\prefback.bmp", 512, 256);
	anykey = new Sprite("anykey.bmp", 256, 32);
	bigtarget = new Sprite("bigtarget.bmp", 256, 256);
	chapters[0] = new Sprite("ass.bmp", 512, 64);
	chapters[1] = new Sprite("doz.bmp", 512, 64);
	chapters[2] = new Sprite("prof.bmp", 512, 64);
}

Preface::~Preface()
{
	for (int i = 0; i < 3; i++) delete chapters[i];
	delete bigtarget;
	delete anykey;
}

void Preface::HandleEvent(HWND, UINT msg, WPARAM, LPARAM)
{
	if ((msg == WM_KEYDOWN || msg == WM_LBUTTONDOWN) && !done) FadeOut();
}

void Preface::Draw()
{
	int i = 0;
	float r = float(0.25 * cos(t * 0.002));
	glTranslatef(0, 0, float(sin(t * 0.002) * 0.25 - 1));
	glBindTexture(GL_TEXTURE_2D, background);
	glBegin(GL_QUADS);
		glTexCoord2i(1, 1);
		glVertex3f(0.7f, 0.5f, 0);
		glTexCoord2i(0, 1);
		glVertex3f(-0.7f, 0.5f, 0);
		glTexCoord1i(0);
		glVertex3f(-0.7f, -0.5f, 0);
		glTexCoord1i(1);
		glVertex3f(0.7f, -0.5f, 0);
	glEnd();
	glLoadIdentity();
	glTranslatef(float(1.5 * r * cos(t * 0.001)), float(r * sin(t * 0.001)), 0);
	bigtarget->Draw(-0.25f, 0.25f, 0.5f, 0.5f);
	glLoadIdentity();
	glEnable(GL_LIGHTING);
	if (!chapter)
	{
		for (; i < NUM_ASS; i++)
		{
			cointex[1] = ass[i];
			DrawCoin(-0.25f + i * 0.16666f, 0, 0.15f, 0.15f, t * 0.05f - 30 * i);
		}
	}
	else if (chapter == 1)
	{
		for (; i < 4; i++)
		{
			cointex[1] = doz[i];
			DrawCoin(-0.25f + i * 0.16666f, 0.07f, 0.15f, 0.15f, t * 0.05f - 30 * i);
		}
		for (i = 0; i < 3; i++)
		{
			cointex[1] = doz[i + 4];
			DrawCoin(-0.16666f + i * 0.16666f, -0.07f, 0.15f, 0.15f, t * 0.05f - 30 * i);
		}
	}
	else
	{
		for (; i < 3; i++)
		{
			cointex[1] = prof[i];
			DrawCoin(-0.16666f + i * 0.16666f, 0.07f, 0.15f, 0.15f, t * 0.05f - 30 * i);
		}
		for (i = 0; i < 2; i++)
		{
			cointex[1] = prof[i + 3];
			DrawCoin(-0.08333f + i * 0.16666f, -0.07f, 0.15f, 0.15f, t * 0.05f - 30 * i);
		}
	}
	glDisable(GL_LIGHTING);
	anykey->Draw(-0.25f, -0.19f, 0.5f, 0.0625f);
	chapters[chapter]->Draw(-0.3f, 0.25f, 0.6f, 0.075f);
	GameObject::Draw();
}

void Preface::DrawCoin(float x, float y, float w, float h, float angle)
{
	float a, b, c, d;
	int i, j;
	glTranslatef(x, y, -1);
	glRotatef(-angle, 0, 1, 0);
	for (i = 0; i < 2; i++)
	{
		glBindTexture(GL_TEXTURE_2D, cointex[i]);
		glBegin(GL_TRIANGLE_FAN);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.5f, 0.5f);
			glVertex3f(0, 0, COIN_WIDTH);
			for (j = 0; j <= COIN_EDGES; j++)
			{
				a = cos_table[j % COIN_EDGES];
				b = sin_table[j % COIN_EDGES];
				glTexCoord2f(a + 0.5f, b + 0.5f);
				glVertex3f(a * w, b * h, COIN_WIDTH);
			}
		glEnd();
		glRotatef(180, 0, 1, 0);
	}
	glBindTexture(GL_TEXTURE_2D, cointex[2]);
	glBegin(GL_QUADS);
		for (j = 0; j <= COIN_EDGES; j++)
		{
			a = cos_table[j % COIN_EDGES];
			b = sin_table[j % COIN_EDGES];
			c = cos_table[(j + 1) % COIN_EDGES];
			d = sin_table[(j + 1) % COIN_EDGES];
			glNormal3f(a, b, 0);
			glTexCoord2i(0, 1);
			glVertex3f(a * w, b * h, COIN_WIDTH);
			glNormal3f(a, b, 0);
			glTexCoord2i(1, 1);
			glVertex3f(a * w, b * h, -COIN_WIDTH);
			glNormal3f(c, d, 0);
			glTexCoord1i(1);
			glVertex3f(c * w, d * h, -COIN_WIDTH);
			glNormal3f(c, d, 0);
			glTexCoord1i(0);
			glVertex3f(c * w, d * h, COIN_WIDTH);
		}
	glEnd();
	glLoadIdentity();
}

void Preface::Start(HWND hWnd)
{
	GameObject::Start(hWnd);
	cointex[0] = rubles[chapter];
	for (int i = 0; i < NUM_DOZ; i++)
		angles[i] = 0;
}

bool Preface::Done(GameObject *&new_obj)
{
	new_obj = shooting;
	return (done && !fade_dir);
}

Shooting::Shooting(): GameObject("shooting")
{
	LoadScene("scenes\\shooting.txt");
	target = LoadTexture("textures\\bigtarget.bmp", 512, 512);
	numbers = LoadTexture("textures\\numbers.bmp", 512, 64);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	ready = new Sprite("ready.bmp", 128, 32);
	mac = new Sprite("mac.bmp", 256, 256);
	mac->z = -0.499f;
	litvinov = new Sprite("mudak.bmp", 256, 256);
	litvinov->z = 0;
	bullet = new Sprite("bullet.bmp", 256, 32);
	paused = new Sprite("paused.bmp", 256, 64);
	mciSendString("open sounds\\fire.wav type waveaudio alias fire", NULL, 0, 0);
	mciSendString("open sounds\\zoom.wav type waveaudio alias zoom", NULL, 0, 0);
	mciSendString("open sounds\\reload.wav type waveaudio alias reload", NULL, 0, 0);
	mciSendString("open sounds\\depleted.wav type waveaudio alias depleted", NULL, 0, 0);
	mciSendString("open sounds\\death1.wav type waveaudio alias death1", NULL, 0, 0);
	mciSendString("open sounds\\death2.wav type waveaudio alias death2", NULL, 0, 0);
}

Shooting::~Shooting()
{
	mciSendString("close fire wait", NULL, 0, 0);
	mciSendString("close zoom wait", NULL, 0, 0);
	mciSendString("close reload wait", NULL, 0, 0);
	mciSendString("close depleted wait", NULL, 0, 0);
	delete paused;
	delete bullet;
	delete litvinov;
	delete mac;
	delete ready;
	delete textures;
	delete edges;
}

void Shooting::LoadScene(char *filename)
{
	int num_tex, i, w, h;
	char name[32], buf[32];
	ifstream in(filename);
	in >> num_edges;
	edges = new Edge[num_edges];
	in >> num_tex;
	textures = new GLuint[num_tex];
	for (i = 0; i < num_tex; i++)
	{
		in >> buf;
		in >> w;
		in >> h;
		wsprintf(name, "textures\\%s.bmp", buf);
		textures[i] = LoadTexture(name, w, h);
	}
	for (i = 0; i < num_edges; i++)
	{
		in >> edges[i].a.x >> edges[i].a.y >> edges[i].a.z;
		in >> edges[i].b.x >> edges[i].b.y >> edges[i].b.z;
		in >> edges[i].c.x >> edges[i].c.y >> edges[i].c.z;
		in >> edges[i].d.x >> edges[i].d.y >> edges[i].d.z;
		in >> edges[i].n.x >> edges[i].n.y >> edges[i].n.z;
		in >> edges[i].ua >> edges[i].va;
		in >> edges[i].ub >> edges[i].vb;
		in >> edges[i].uc >> edges[i].vc;
		in >> edges[i].ud >> edges[i].vd;
		in >> w;
		edges[i].tex = textures[w];
		in >> w >> h;
		in >> edges[i].lua >> edges[i].lva;
		in >> edges[i].lub >> edges[i].lvb;
		in >> edges[i].luc >> edges[i].lvc;
		in >> edges[i].lud >> edges[i].lvd;
		wsprintf(name, "lightmaps\\%d.bmp", i);
		edges[i].lightmap = LoadTexture(name, w, h);
	}
	in.close();
}

void Shooting::DrawStaticScene()
{
	Edge *e = edges;
	for (int i = 0; i < num_edges; i++, e++)
	{
		glBindTexture(GL_TEXTURE_2D, e->tex);
		glBegin(GL_QUADS);
			glNormal3f(e->n.x, e->n.y, e->n.z);
			glTexCoord2f(e->ua, e->va);
			glVertex3f(e->a.x, e->a.y, e->a.z);
			glTexCoord2f(e->ub, e->vb);
			glVertex3f(e->b.x, e->b.y, e->b.z);
			glTexCoord2f(e->uc, e->vc);
			glVertex3f(e->c.x, e->c.y, e->c.z);
			glTexCoord2f(e->ud, e->vd);
			glVertex3f(e->d.x, e->d.y, e->d.z);
		glEnd();
		glBindTexture(GL_TEXTURE_2D, e->lightmap);
		glEnable(GL_BLEND);
		glBlendFunc(GL_DST_COLOR, GL_ZERO);
		glBegin(GL_QUADS);
			glNormal3f(e->n.x, e->n.y, e->n.z);
			glTexCoord2f(e->lua, e->lva);
			glVertex3f(e->a.x, e->a.y, e->a.z);
			glTexCoord2f(e->lub, e->lvb);
			glVertex3f(e->b.x, e->b.y, e->b.z);
			glTexCoord2f(e->luc, e->lvc);
			glVertex3f(e->c.x, e->c.y, e->c.z);
			glTexCoord2f(e->lud, e->lvd);
			glVertex3f(e->d.x, e->d.y, e->d.z);
		glEnd();
		glDisable(GL_BLEND);
	}
}

void Shooting::DrawTime()
{
	int n[4], i;
	float x, u;
	n[0] = timeout / 60000;
	n[1] = 10;
	n[2] = (timeout % 60000) / 10000;
	n[3] = (timeout % 10000) / 1000;
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);
//	glBlendFunc(GL_SRC_COLOR, GL_DST_COLOR);
	glBindTexture(GL_TEXTURE_2D, numbers);
	glBegin(GL_QUADS);
		for (i = 0; i < 4; i++)
		{
			x = 0.9f + i * 0.085f;
			u = n[i] * 0.0625f + 0.001f;
			glTexCoord2f(u, 1);
			glVertex3f(x, 0.9f, -1);
			glTexCoord1f(u);
			glVertex3f(x, 0.75f, -1);
			glTexCoord1f(u + 0.06f);
			glVertex3f(x + 0.075f, 0.75f, -1);
			glTexCoord2f(u + 0.06f, 1);
			glVertex3f(x + 0.075f, 0.9f, -1);
		}
	glEnd();
	glDisable(GL_BLEND);
}

void Shooting::DrawShells()
{
	for (int i = 0; i < shells; i++)
		bullet->Draw(-1.25f, -0.87f + i * 0.06f, 0.56f, 0.07f);
}

void Shooting::Draw()
{
	int i;
	float z = zoomed ? 1 / zoom_level : 1;
	float a = reloading ? float(exp(2.5 * pow(0.0004 * reloading, 16)) - 1) : 0;
	glEnable(GL_DEPTH_TEST);
	glScalef(0.5, 0.5, z);
	glRotatef(a_x - a, 1, 0, 0);
	glRotatef(a_y, 0, 1, 0);
	glTranslatef(-0.5, -0.25, 0.25);
	DrawStaticScene();
	for (i = 0; i < num_targets; i++)
		targets[i].Draw();
	for (i = 0; i < 3; i++)
		mac->Draw(0.25f + i * 0.166666f, 0.5, 0.166666f, 0.166666f);
	glTranslatef(0.01f, 0, 0);
	glRotatef(90, 0, 1, 0);
	litvinov->Draw(-0.25, 0.5, 0.5, 0.5);
	glDisable(GL_DEPTH_TEST);
	glLoadIdentity();
	glScalef(0.28f, 0.28f, 1);
	if (zoomed)
	{
		glEnable(GL_BLEND);
		glBlendFunc(GL_DST_COLOR, GL_ZERO);
		glBindTexture(GL_TEXTURE_2D, target);
		glBegin(GL_QUADS);
			glTexCoord1i(0);
			glVertex3i(-1, 1, -1);
			glTexCoord2i(0, 1);
			glVertex3i(-1, -1, -1);
			glTexCoord2i(1, 1);
			glVertex3i(1, -1, -1);
			glTexCoord1i(1);
			glVertex3i(1, 1, -1);
		glEnd();
		glDisable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);
		glColor3f(0, 0, 0);
		glBegin(GL_QUADS);
			glVertex3i(-2, 1, -1);
			glVertex3i(-2, -1, -1);
			glVertex3i(-1, -1, -1);
			glVertex3i(-1, 1, -1);
			glVertex3i(1, 1, -1);
			glVertex3i(1, -1, -1);
			glVertex3i(2, -1, -1);
			glVertex3i(2, 1, -1);
		glEnd();
		if (reloading)
		{
			glEnable(GL_BLEND);
			glBlendFunc(GL_DST_COLOR, GL_ONE);
			a = reloading > 2450 ? 1.0f : 0.0f;
			glColor3f(a, a, a);
			glBegin(GL_QUADS);
				glVertex3i(-2, 1, -1);
				glVertex3i(-2, -1, -1);
				glVertex3i(2, -1, -1);
				glVertex3i(2, 1, -1);
			glEnd();
			glDisable(GL_BLEND);
		}
		glColor3f(1, 1, 1);
		glEnable(GL_TEXTURE_2D);
		if (!reloading && ((t % 200) > 100) && shells) ready->Draw(0.9f, -0.85f, 0.4f, 0.1f);
		DrawShells();
	}
	if (timeout > 10000 || ((t % 200) > 100)) DrawTime();
	if (pause) paused->Draw(-0.5, 0.125, 1, 0.25);
	GameObject::Draw();
}

void Shooting::Start(HWND hWnd)
{
	GLuint *list;
	GameObject::Start(hWnd);
	a_x = 0;
	a_y = 0;
	zoomed = false;
	zoom_level = 2;
	zoom_dir = 0;
	reloading = 0;
	hit = 0;
	pause = false;
	next = NULL;
	switch (chapter)
	{
		case 0: 
			target_speed = 0.0005f;
			list = ass;
			num_targets = NUM_ASS; 
			timeout = 60000; 
			shells = 10; 
			break;
		case 1: 
			target_speed = 0.001f;
			list = doz;
			num_targets = NUM_DOZ; 
			timeout = 45000; 
			shells = 10; 
			break;
		case 2: 
			target_speed = 0.0007f;
			list = prof;
			num_targets = NUM_PROF; 
			timeout = 30000; 
			shells = NUM_PROF;
	}
	width = num_targets * 0.5f - 0.3f;
	targets = new Target[num_targets];
	for (int i = 0; i < num_targets; i++)
	{
		targets[i].angle = 0;
		targets[i].tex = list[i];
		targets[i].x = float(0.6 - width * 0.5 + i * 0.5);
		targets[i].y = 0.1f;
		targets[i].z = -3.8f;
	}
	CenterMouse(hWnd);
}

void Shooting::Stop()
{
	GameObject::Stop();
	mciSendString("stop fire wait", NULL, 0, 0);
	mciSendString("seek fire to start wait", NULL, 0, 0);
	mciSendString("stop zoom wait", NULL, 0, 0);
	mciSendString("seek zoom to start wait", NULL, 0, 0);
	mciSendString("stop reload wait", NULL, 0, 0);
	mciSendString("seek reload to start wait", NULL, 0, 0);
	mciSendString("stop depleted wait", NULL, 0, 0);
	mciSendString("seek depleted to start wait", NULL, 0, 0);
	mciSendString("stop death1 wait", NULL, 0, 0);
	mciSendString("seek death1 to start wait", NULL, 0, 0);
	mciSendString("stop death2 wait", NULL, 0, 0);
	mciSendString("seek death2 to start wait", NULL, 0, 0);
}

void Shooting::HandleEvent(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	RECT r;
	int i;
	float p = zoomed ? 1 / zoom_level : 1;
	GameObject::HandleEvent(hWnd, msg, wParam, lParam);
	switch (msg)
	{
		case WM_MOUSEMOVE:
			if (pause) break;
			GetClientRect(hWnd, &r);
			a_y += MOUSE_SENS * p * (LOWORD(lParam) - ((r.left + r.right) >> 1));
			a_x += MOUSE_SENS * p * (HIWORD(lParam) - ((r.top + r.bottom) >> 1));
			CenterMouse(hWnd);
			if (a_x < -90) a_x = -90;
			if (a_x > 90) a_x = 90;
			if (a_y < -360) a_y += 360;
			if (a_y > 360) a_y -= 360;
			break;

		case WM_LBUTTONDOWN:
			if (!zoomed || reloading || pause) break;
			if (!shells)
			{
				mciSendString("seek depleted to start wait", NULL, 0, 0);
				mciSendString("play depleted notify", NULL, 0, hWnd);
				break;
			}
			for (i = 0; i < num_targets; i++)
				if (!targets[i].dead)
					if (targets[i].Shoot(Vector(0.5, 0.25, -0.25), a_x, a_y))
					{
						hit++;
						if (chapter != 2)
							mciSendString("play death1 notify", NULL, 0, hWnd);
						else
							mciSendString("play death2 notify", NULL, 0, hWnd);
						if (hit == num_targets)
						{
							next = preface;
							FadeOut();
						}
						break;
					}
			mciSendString("play fire notify", NULL, 0, hWnd);
			reloading = 2500;
			rel = false;
			shells--;
			break;

		case WM_RBUTTONDOWN:
			if (pause) break;
			zoomed ^= true;
			mciSendString("play zoom notify", NULL, 0, hWnd);
			break;

		case WM_KEYDOWN:
			switch (unsigned(wParam))
			{
				case VK_PAUSE:
					if (done) break;
					pause ^= true;
					if (pause) old_t = t;
					else t = timeGetTime() - time;
					break;
				case VK_ESCAPE:
					next = menu;
					if (!done) FadeOut();
			}
			if (zoomed && !pause)
			switch (unsigned(wParam))
			{
				case 0x41: zoom_dir = 1; break;
				case 0x5A: zoom_dir = -1; break;
			}
			break;

		case WM_KEYUP:
			if (zoomed)
			switch (unsigned(wParam))
			{
				case 0x41: if (zoom_dir > 0) zoom_dir = 0; break;
				case 0x5A: if (zoom_dir < 0) zoom_dir = 0;
			}
	}
}

void Shooting::UpdateObject(HWND hWnd)
{
	GameObject::UpdateObject(hWnd);
	if (pause)
	{
		t = old_t;
		dt = 0;
	}
	if (timeout < dt) timeout = 0;
	else timeout -= dt;
	if (reloading > 0) 
	{
		reloading -= dt;
		if (reloading < 1500 && !rel && shells) 
		{
			mciSendString("play reload notify", NULL, 0, hWnd);
			rel = true;
		}
	}
	if (reloading < 0) reloading = 0;
	if (!timeout && !done && !next) { next = gameover; FadeOut(); }
	zoom_level *= float(pow(2.0, zoom_dir * double(dt) * 0.002));
	if (zoom_level > 8) zoom_level = 8;
	if (zoom_level < 2) zoom_level = 2;
	for (int i = 0; i < num_targets; i++)
	{
		if (targets[i].y < 0.375f && chapter != 2)
		{
			targets[i].y += dt * 0.0001f;
			if (targets[i].y > 0.375f) targets[i].y = 0.375f;
		}
		else
		{
			targets[i].x += dt * target_speed;
			if (targets[i].x > float(width * 0.5 + 0.9)) targets[i].x = float(0.6 - width * 0.5);
			if (chapter == 2) targets[i].y = float(0.375 + sin(t * 0.005 + i * M_PI / 3) * 0.075);
		}
		if (targets[i].dead)
		{
			targets[i].angle += dt * 0.2f;
			if (targets[i].angle > 90) targets[i].angle = 90;
		}
	}
}

bool Shooting::Done(GameObject *&new_obj)
{
	if (done && !fade_dir)
	{
		if (chapter == 2 && next == preface)
			new_obj = wingame;
		else
		{
			new_obj = next;
			chapter++;
		}
		return true;
	}
	return false;
}

WinGame::WinGame(): GameObject("wingame")
{
	int i;
	LoadScene("scenes\\teapot.txt");
	background = LoadTexture("textures\\all.bmp", 512, 512);
	pal = new unsigned char[1024];
	index_buf = new unsigned char[16384];
	rgb_buf = new unsigned char[49152];
	for (i = 0; i < 64; i++)
	{
		pal[i << 2] = i << 2;
		pal[(i << 2) + 1] = 0;
		pal[(i << 2) + 2] = 0;
		pal[(i + 64) << 2] = 255;
		pal[((i + 64) << 2) + 1] = i << 2;
		pal[((i + 64) << 2) + 2] = 0;
	}
	for (i = 0; i < 128; i++)
	{
		pal[(i + 128) << 2] = 255;
		pal[((i + 128) << 2) + 1] = 255;
		pal[((i + 128) << 2) + 2] = i << 1;
	}
	mciSendString("open sounds\\wingame.wav alias wingame", NULL, 0, 0);
}

WinGame::~WinGame()
{
	delete rgb_buf;
	delete index_buf;
	delete pal;
	delete f;
	delete v;
}

void WinGame::LoadScene(char *filename)
{
	int i;
	ifstream in(filename);
	
	in >> num_v >> num_f;
	v = new Vertex[num_v];
	f = new Face[num_f];

	for (i = 0; i < num_v; i++)
	{
		in >> v[i].x >> v[i].y >> v[i].z;
		in >> v[i].nx >> v[i].ny >> v[i].nz;
	}

	for (i = 0; i < num_f; i++)
		in >> f[i].a >> f[i].b >> f[i].c;

	in.close();
}

void WinGame::DrawScene()
{
	glBegin(GL_TRIANGLES);
		for (int i = 0; i < num_f; i++)
		{
			glNormal3f(v[f[i].a].nx, v[f[i].a].ny, v[f[i].a].nz);
			glVertex3f(v[f[i].a].x, v[f[i].a].y, v[f[i].a].z);
			glNormal3f(v[f[i].b].nx, v[f[i].b].ny, v[f[i].b].nz);
			glVertex3f(v[f[i].b].x, v[f[i].b].y, v[f[i].b].z);
			glNormal3f(v[f[i].c].nx, v[f[i].c].ny, v[f[i].c].nz);
			glVertex3f(v[f[i].c].x, v[f[i].c].y, v[f[i].c].z);
		}
	glEnd();
}

void WinGame::DrawFire()
{
	int i = 0, k = 0;

	for (; i < 16384; i++, k += 3)
		memcpy(rgb_buf + k, pal + (((unsigned int)index_buf[i]) << 2), 3);
	
	glBindTexture(GL_TEXTURE_2D, GetTexture(NUM_TEXTURES - 1));
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

	glTexImage2D(GL_TEXTURE_2D, 0, 3, 128, 128, 0,
        	GL_RGB, GL_UNSIGNED_BYTE, rgb_buf);

	glScalef(0.4f, 0.3f, 1);
	glBegin(GL_QUADS);
		glTexCoord1i(0);
		glVertex3i(-1, 1, -1);
		glTexCoord2i(0, 1);
		glVertex3i(-1, -1, -1);
		glTexCoord2i(1, 1);
		glVertex3i(1, -1, -1);
		glTexCoord1i(1);
		glVertex3i(1, 1, -1);
	glEnd();
}

void WinGame::Draw()
{
	float p = float(sin(t * 0.001) * 0.5 + 0.5);
	GLfloat diffuse1[] = {0.25, 1, p};
	GLfloat diffuse2[] = {1, p, 0};
	GLfloat diffuse3[] = {0, p, 1 - p};
	float a = t * 0.05f;

	DrawFire();

	glLoadIdentity();
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glTranslatef(0, -0.3f, -10);
	glRotatef(-60, 1, 0, 0);
	glRotatef(a, 0, 0, 1);
	glPushMatrix();
	glScalef(4, 4, 1);
	glTranslatef(-0.5, -0.5, 0);
	glBindTexture(GL_TEXTURE_2D, background);
	glBegin(GL_QUADS);
		glTexCoord2i(0, 1);
		glVertex2i(0, 1);
		glTexCoord1i(0);
		glVertex2i(0, 0);
		glTexCoord1i(1);
		glVertex2i(1, 0);
		glTexCoord2i(1, 1);
		glVertex2i(1, 1);
	glEnd();
	glEnable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
		glNormal3f(-1, 0, 0);
		glVertex2i(0, 0);
		glVertex3i(0, 1, 0);
		glVertex3f(0, 1, -0.1f);
		glVertex3f(0, 0, -0.1f);
		glNormal3f(1, 0, 0);
		glVertex2i(1, 0);
		glVertex3f(1, 0, -0.1f);
		glVertex3f(1, 1, -0.1f);
		glVertex3i(1, 1, 0);
		glNormal3f(0, -1, 0);
		glVertex2i(0, 0);
		glVertex3f(0, 0, -0.1f);
		glVertex3f(1, 0, -0.1f);
		glVertex3i(1, 0, 0);
		glNormal3f(0, 1, 0);
		glVertex2i(0, 1);
		glVertex3i(1, 1, 0);
		glVertex3f(1, 1, -0.1f);
		glVertex3f(0, 1, -0.1f);
	glEnd();
	glEnable(GL_DEPTH_TEST);
	glPopMatrix();	
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse1);
	DrawScene();
	glLoadIdentity();
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse2);
	glTranslatef(-2.5f, 2.2f, -10);
	glRotatef(float(cos(a * 0.05) * 30), 1, 0, 0);
	glTranslatef(0, -0.4f, 0.1f);
	glListBase(1000);
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glCallLists(11, GL_UNSIGNED_BYTE, "IT shooting");
	glPopAttrib();
	glLoadIdentity();
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse3);
	glTranslatef(-2.9f, -2, -10);
	glRotatef(float(sin(a * 0.1) * 30), 1, 0, 0);
	glTranslatef(0, -0.4f, 0.1f);
	glListBase(1000);
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glCallLists(13, GL_UNSIGNED_BYTE, "     Victory!");
	glPopAttrib();
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glPopAttrib();

	GameObject::Draw();
}

void WinGame::Start(HWND hWnd)
{
	GameObject::Start(hWnd);
	ZeroMemory(index_buf, 16384);
	mciSendString("play wingame notify", NULL, 0, hWnd);
}

void WinGame::Stop()
{
	GameObject::Stop();
	mciSendString("seek wingame to start wait", NULL, 0, 0);
}

void WinGame::HandleEvent(HWND, UINT msg, WPARAM, LPARAM)
{
	if ((msg == WM_KEYDOWN || msg == WM_LBUTTONDOWN) && !done) FadeOut();
}

void WinGame::UpdateFire()
{
	int i = 0;
	unsigned short color;
	unsigned char *u = index_buf;

	for (; i < 256; i++)
		index_buf[16128 + i] = 255 * rand() / RAND_MAX;

	for (i = 0; i < 16128; i++, u++)
	{
		color = u[127] + u[128] + u[129] + u[256];
		color >>= 2;
		if (color) color--;
		*u = (unsigned char)color;
	}
}

void WinGame::UpdateObject(HWND hWnd)
{
	GameObject::UpdateObject(hWnd);
	UpdateFire();
}

bool WinGame::Done(GameObject *&new_obj)
{
	new_obj = menu;
	return (done && !fade_dir);
}

GameOver::GameOver(): GameObject("gameover")
{
	background = LoadTexture("textures\\asm.bmp", 512, 256);
	mciSendString("open sounds\\gameover.wav alias gameover", NULL, 0, 0);
}

GameOver::~GameOver()
{
}

void GameOver::Draw()
{
	float offs = t * -0.001f;
	float a = t * 0.1f, x = 0, y = 0;
	int i;
	GLfloat diffuse[2][3] = {{1, 1, 0}, {0, 1, 1}};
	GLYPHMETRICSFLOAT m;

	glColor3f(0.5, 0.5, 0.5);
	glBindTexture(GL_TEXTURE_2D, background);
	glBegin(GL_QUADS);
		glTexCoord2f(0, 2 + offs);
		glVertex3f(-0.3f, 0.4f, -1);
		glTexCoord2f(0, offs);
		glVertex3f(-0.3f, -0.4f, -1);
		glTexCoord2f(1, offs);
		glVertex3f(0.5f, -0.4f, -1);
		glTexCoord2f(1, 2 + offs);
		glVertex3f(0.5f, 0.4f, -1);
	glEnd();
	glColor3i(1, 1, 1);

	glDisable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	for (i = 0; i < 8; i++)
	{
		glMaterialfv(GL_FRONT, GL_DIFFUSE, (i + (i < 4)) & 1 ? diffuse[0] : diffuse[1]);
		m = agmf[overstr[i]];
		glPushMatrix();
		glTranslatef(x - 1.05f, 0.4f - y, -5); 
		glRotatef(a + i * 15, 0, 1, 0);
		glTranslatef(-m.gmfptGlyphOrigin.x - m.gmfBlackBoxX / 2, -m.gmfptGlyphOrigin.y + m.gmfBlackBoxY / 2, 0.1f); 
		x += m.gmfBlackBoxX + 0.1f;
		glCallList(1000 + overstr[i]); 
		glPopMatrix();
		if (i == 3)
		{
			x = 0.1f;
			y += 0.8f;
		}
	}
	glPopAttrib();
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);

	GameObject::Draw();
}

bool GameOver::Done(GameObject *&new_obj)
{
	new_obj = menu;
	return (done && !fade_dir);
}

void GameOver::HandleEvent(HWND, UINT msg, WPARAM, LPARAM)
{
	if ((msg == WM_KEYDOWN || msg == WM_LBUTTONDOWN) && !done) FadeOut();
}

void GameOver::Start(HWND hWnd)
{
	GameObject::Start(hWnd);
	mciSendString("play gameover notify", NULL, 0, hWnd);
}

void GameOver::Stop()
{
	GameObject::Stop();
	mciSendString("seek gameover to start wait", NULL, 0, 0);
}

Target::Target()
{
	dead = false;
}

void Target::Draw()
{
	if (angle >= 90) return;
	float a, b;
	glPushMatrix();
	glTranslatef(x, y, z);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTranslatef(0, -0.2f, 0);
	glRotatef(-angle, 1, 0, 0);
	glTranslatef(0, 0.2f, 0);
	glScalef(0.05f, 0.05f, 1);
	glBegin(GL_TRIANGLE_FAN);
		glTexCoord2f(0.5f, 0.5f);
		glVertex3f(0, 0, COIN_WIDTH);
		for (int i = 0; i <= COIN_EDGES; i++)
		{
			a = cos_table[i % COIN_EDGES];
			b = sin_table[i % COIN_EDGES];
			glTexCoord2f(a + 0.5f, b + 0.5f);
			glVertex2f(2 * a, 2 * b);
		}
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
		glColor3f(0.8f, 0.8f, 0.8f);
		glNormal3f(0, 0, 1);
		glVertex3f(-0.1f, -0.9f, 0.005f);
		glVertex3f(-0.1f, -4, 0.005f);
		glVertex3f(0.1f, -4, 0.005f);
		glVertex3f(0.1f, -0.9f, 0.005f);
		glColor3f(0.4f, 0.4f, 0.4f);
		glNormal3f(-1, 0, 0);
		glVertex3f(-0.1f, -0.9f, 0.005f);
		glVertex3f(-0.1f, -0.9f, -0.005f);
		glVertex3f(-0.1f, -4, -0.005f);
		glVertex3f(-0.1f, -4, 0.005f);
		glColor3f(0.4f, 0.4f, 0.4f);
		glNormal3f(1, 0, 0);
		glVertex3f(0.1f, -0.9f, 0.005f);
		glVertex3f(0.1f, -4, 0.005f);
		glVertex3f(0.1f, -4, -0.005f);
		glVertex3f(0.1f, -0.9f, -0.005f);
	glEnd();
	glColor3f(1, 1, 1);
	glEnable(GL_TEXTURE_2D);
	glPopMatrix();
}

bool Target::Shoot(Vector v, float a_x, float a_y)
{
	if (a_x == 90 || a_x == -90 || (cos(PI180 * a_y) < 0.989949493f)) return false;
	float d = v.z - z;
	float x_ = v.x + float(d * tan(PI180 * a_y));
	float y_ = v.y + float(d * tan(-PI180 * a_x));
	if (y_ < 0.25) return false;
	float dx = x_ - x;
	float dy = y_ - y;
	if (sqrt(dx * dx + dy * dy) > 0.05) return false;
	dead = true;
	return true;
}

Intro::Intro(): GameObject("intro")
{
	int i, j;
	float c1 = 3.0f / NUM_VERTEX_Y;
	float c2 = 4.0f / NUM_VERTEX_X;
	strcpy(str, "IT shooting");
	l = strlen(str);
	w = 0;
	for (i = 0; i < l; i++)
		w += agmf[str[i]].gmfBlackBoxX + 0.1f;
	w -= 0.1f;
	v = new Vertex *[NUM_VERTEX_Y];
	for (i = 0; i < NUM_VERTEX_Y; i++)
	{
		v[i] = new Vertex[NUM_VERTEX_X];
		for (j = 0; j < NUM_VERTEX_X; j++)
		{
			v[i][j].x = -2 + j * c2;
			v[i][j].y = -1.5f + i * c1;
			v[i][j].z = 0;
			v[i][j].nx = 0;
			v[i][j].ny = 0;
			v[i][j].nz = 1;
		}
	}
	background = LoadTexture("textures\\intro.bmp", 512, 512);
}

Intro::~Intro()
{
	int i = 0;
	for (; i < NUM_VERTEX_Y; i++)
		delete v[i];
	delete v;
}

void Intro::Draw()
{
	float a1, a2, x = 0;
	float y = -agmf[str[0]].gmfptGlyphOrigin.y + agmf[str[0]].gmfBlackBoxY / 2;
	float c1, c2, u, v_ = -0.25;
	float du = 2.0f / NUM_VERTEX_X;
	float dv = 1.5f / NUM_VERTEX_Y;
	int i, j;
	GLfloat diffuse[] = {1, 0, 0};
	GLYPHMETRICSFLOAT m;

	glEnable(GL_LIGHTING);
	glTranslatef(0, 0, -4);
	glBindTexture(GL_TEXTURE_2D, background);
	for (i = 0; i < NUM_VERTEX_Y - 1; i++, v_ += dv)
	{
		u = 0.5;
		for (j = 0; j < NUM_VERTEX_X - 1; j++, u += du)
		{
			if (j > 19 && j < 60 && i > 9 && i < 50) glEnable(GL_TEXTURE_2D);
			else glDisable(GL_TEXTURE_2D);
			glBegin(GL_TRIANGLES);
				glTexCoord2f(u, v_);
				glNormal3fv((float *)&v[i][j].nx);
				glVertex3fv((float *)&v[i][j]);
				glTexCoord2f(u + du, v_);
				glNormal3fv((float *)&v[i][j + 1].nx);
				glVertex3fv((float *)&v[i][j + 1]);
				glTexCoord2f(u, v_ + dv);
				glNormal3fv((float *)&v[i + 1][j].nx);
				glVertex3fv((float *)&v[i + 1][j]);
				glTexCoord2f(u + du, v_ + dv);
				glNormal3fv((float *)&v[i + 1][j + 1].nx);
				glVertex3fv((float *)&v[i + 1][j + 1]);
				glTexCoord2f(u, v_ + dv);
				glNormal3fv((float *)&v[i + 1][j].nx);
				glVertex3fv((float *)&v[i + 1][j]);
				glTexCoord2f(u + du, v_);
				glNormal3fv((float *)&v[i][j + 1].nx);
				glVertex3fv((float *)&v[i][j + 1]);
			glEnd();
		}
	}
	glLoadIdentity();
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	for (i = 0; i < l; i++)
	{
		c1 = float(cos(t * 0.005 + i * M_PI / l));
		c2 = float(sin(t * 0.005 + i * M_PI / l));
		if (c1 >= 0)
		{
			if (c2 >= 0)
			{
				diffuse[0] = c2;
				diffuse[1] = c1;
				diffuse[2] = 0;
			}
			else
			{
				diffuse[0] = 0;
				diffuse[1] = c1;
				diffuse[2] = -c2;
			}
		}
		else
		{
			if (c2 >= 0)
			{
				diffuse[0] = 1;
				diffuse[1] = -c1;
				diffuse[2] = -c1;
			}
			else
			{
				diffuse[0] = -c1;
				diffuse[1] = -c1;
				diffuse[2] = 1;
			}
		}
		a1 = float(sin(t * 0.005 + i * M_PI / 12) * 20);
		a2 = float(cos(t * 0.01 + i * M_PI / 12) * 20);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
		m = agmf[str[i]];
		glPushMatrix();
		glTranslatef(x - w / 2, y, float(-8 + sin(t * 0.005 + i * M_PI / 6) * 0.5)); 
		glTranslatef(m.gmfptGlyphOrigin.x + m.gmfBlackBoxX / 2, 0, 0); 
		glRotatef(a1, 0, 1, 0);
		glRotatef(a2, 1, 0, 0);
		glTranslatef(-m.gmfptGlyphOrigin.x - m.gmfBlackBoxX / 2, 0, 0.1f); 
		x += m.gmfBlackBoxX + 0.1f;
		glCallList(1000 + str[i]); 
		glPopMatrix();
	}
	glPopAttrib();
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);

	GameObject::Draw();
}

bool Intro::Done(GameObject *&new_obj)
{
	new_obj = menu;
	return (done && !fade_dir);
}

void Intro::HandleEvent(HWND, UINT msg, WPARAM, LPARAM)
{
	if ((msg == WM_KEYDOWN || msg == WM_LBUTTONDOWN) && !done) FadeOut();
}

void Intro::UpdateObject(HWND hWnd)
{
	GameObject::UpdateObject(hWnd);
	int i = 0, j;
	float p, q = t * 0.001f;
	for (; i < NUM_VERTEX_Y; i++)
		for (j = 0; j < NUM_VERTEX_X; j++)
		{
			p = float(cos(v[i][j].x * M_PI + q) * cos(v[i][j].y * M_PI + q) * 0.25);
			v[i][j].z = p;
			v[i][j].nx = 0;
			v[i][j].ny = 0;
			v[i][j].nz = p + 0.75f;
		}
}

