#ifndef _CLASSES_H_
#define _CLASSES_H_

#include <gl\gl.h>

#define COIN_EDGES		32
#define COIN_WIDTH		0.005f

#define NUM_ASS			4
#define NUM_DOZ			7
#define NUM_PROF		5

#define MOUSE_SENS		0.2f

#define FADE_TIME		0.002f

#define NUM_VERTEX_X	80
#define NUM_VERTEX_Y	60

class GameObject {
	MCIDEVICEID midi_id;
	float alpha;
protected:
	int fade_dir;
	bool done;
	DWORD time, t, dt;
	void FadeOut();
	void FadeIn();
public:
	GameObject(char *theme);
	~GameObject();
	virtual void HandleEvent(HWND, UINT, WPARAM, LPARAM);
	virtual void Draw();
	virtual void Start(HWND);
	virtual void Stop();
	virtual void UpdateObject(HWND);
	virtual bool Done(GameObject *&) = 0;
};

class Sprite {
	GLuint sprite, mask;
public:
	Sprite(char *, int, int);
	void Draw(float, float, float, float);
	float t, z;
};

class Menu: public GameObject {
	GameObject *next;
	Sprite *pointer;
	GLuint background;
	int current_item;
	float x, y, y1, y2;
	void DrawString(float, float, char *, bool);
public:
	Menu();
	~Menu();
	void HandleEvent(HWND, UINT, WPARAM, LPARAM);
	void Draw();
	void UpdateObject(HWND);
	void Start(HWND);
	void Stop();
	bool Done(GameObject *&);
};

class Preface: public GameObject {
	float angles[NUM_DOZ];
	GLuint rubles[3];
	GLuint cointex[3];
	GLuint background;
	Sprite *anykey;
	Sprite *bigtarget;
	Sprite *chapters[3];
	void DrawCoin(float, float, float, float, float);
public:
	Preface();
	~Preface();
	void HandleEvent(HWND, UINT, WPARAM, LPARAM);
	void Draw();
	void Start(HWND);
	bool Done(GameObject *&);
};

struct Vector {
	float x, y, z;
	Vector() { }
	Vector(float x_, float y_, float z_)
	{
		x = x_;
		y = y_;
		z = z_;
	}
};

struct Edge {
	Vector a, b, c, d;
	Vector n;
	float ua, ub, uc, ud;
	float va, vb, vc, vd;
	float lua, lub, luc, lud;
	float lva, lvb, lvc, lvd;
	GLuint tex;
	GLuint lightmap;
};

struct Target {
	bool dead;
	float x, y, z;
	GLuint tex;
	float angle;
	Target();
	void Draw();
	bool Shoot(Vector, float, float);
};

class Shooting: public GameObject {
	Edge *edges;
	GLuint *textures;
	int num_edges;
	void DrawStaticScene();
	void DrawTime();
	void DrawShells();
	void LoadScene(char *);
	float a_x, a_y;
	bool zoomed, rel;
	float zoom_level;
	int zoom_dir;
	int reloading;
	GLuint target;
	GLuint numbers;
	Sprite *ready;
	Sprite *mac;
	Sprite *litvinov;
	Sprite *bullet;
	Sprite *paused;
	DWORD timeout;
	int shells;
	GameObject *next;
	int num_targets;
	Target *targets;
	float width;
	float target_speed;
	int hit;
	bool pause;
	DWORD old_t;
public:
	Shooting();
	~Shooting();
	void HandleEvent(HWND, UINT, WPARAM, LPARAM);
	void Draw();
	void Start(HWND);
	void Stop();
	void UpdateObject(HWND);
	bool Done(GameObject *&);
};

struct Vertex {
	float x, y, z;
	float nx, ny, nz;
};

struct Face {
	int a, b, c;
};

class WinGame: public GameObject{
	Vertex *v;
	Face *f;
	GLuint background;
	int num_v, num_f;
	void LoadScene(char *);
	void DrawScene();
	void UpdateFire();
	void DrawFire();
	unsigned char *pal;
	unsigned char *index_buf;
	unsigned char *rgb_buf;
public:
	WinGame();
	~WinGame();
	void HandleEvent(HWND, UINT msg, WPARAM, LPARAM);
	void Draw();
	void Start(HWND);
	void Stop();
	void UpdateObject(HWND);
	bool Done(GameObject *&);
};

class GameOver: public GameObject {
	GLuint background;
	HWND handle;
public:
	GameOver();
	~GameOver();
	void HandleEvent(HWND, UINT, WPARAM, LPARAM);
	void Draw();
	bool Done(GameObject *&);
	void Start(HWND);
	void Stop();
};

class Intro: public GameObject {
	Vertex **v;
	float w;
	char str[16];
	int l;
	GLuint background;
public:
	Intro();
	~Intro();
	void Draw();
	bool Done(GameObject *&);
	void HandleEvent(HWND, UINT, WPARAM, LPARAM);
	void UpdateObject(HWND);
};

GameObject *CreateObjects();
void DeleteObjects();
void CenterMouse(HWND);

#endif
